#ifndef COLINTERFACE_H
#define COLINTERFACE_H
/* colinterface.h 
     Interface to the collaborative filtering system
     by Dave Maltz 10/93

  history:
  10/25/93 created

*/

/* get the configuration parameters from the user digestable collabconfig.h */
#include "collabconfig.h"

/********* Exported structures **********/

struct _vote_data_t {
  int 			num_high;
  int			num_med;
  int			num_low;
  int			num_terrible;
  char 			*stuff;
};
typedef struct _vote_data_t vote_data_t;
    

struct _vote_t {
  struct _vote_t	*next_vote;
  vote_data_t		*vote_data;
  char			*message_id;
  void			*internal_magic;
};
typedef struct _vote_t vote_t;

/********* Exported routines ************/

/* return status: 0 = failed, 1 = successful */
int collab_enter_vote_anonymously(/* char *message_id, vote, char *group_name */);
int collab_enter_vote(/* char *message_id, vote, char *group_name */);
int collab_send_off_votes(/* char *vote_file */); 

/* return a new string of the group names sepperated by \n characters;
   the number of groups will be left in the loc pointed to by num_groups */
char* collab_available_groups(/* int *num_groups */);

/* return NULL if there are no votes available, otherwise a list of votes */
vote_t* collab_get_votes_for_group(/* char *group_name */);

/* return status: 0 = failed, 1 = successful;
   arguements will contain the statistics extracted from the entire database 
   of votes */
int collab_get_statistics(/* int *num_votes, double *mean, double *stnd_dev */);

/* change the list of sources we collect data from */
int collab_add_source(/* char *source_name */);
int collab_remove_source(/* char *source_name */);

/* free memory for the votes */
void collab_destroy_votes(/* vote *votes */);

/* notify someone of problems in the collaborative system */
int issue_error(/* varargs : fmt, args */);

/********* Exported constants **********/
#define NAME_SIZE 100    /* the largest string (name) that a user can assoc with
			    a vote */

#define HIGH_VOTE 4
#define MED_VOTE 2
#define LOW_VOTE 1
#define DEFAULT_VOTE 0
#define HATEIT_VOTE -1

#endif /*  COLINTERFACE_H */
