/* routines for working with the data abstractions in the collaborative interface
     by Dave Maltz

 history:
 10/25/93 created

*/

#include "collabdata.h"
#include "colinterface.h"
#include "colinternal.h"

#include <string.h>
#include <stdio.h>

/*========================================================================
  Parsing routines for the vote structures
------------------------------------------------------------------------*/
char intscratch[13];   /* enough space to write 2^32 in decimal */
int
  collab_parse_vote_into_string(buf, buflen, vote, size )
char *buf; 
int buflen;
vote_t *vote;
int *size;
/* take a vote and turn it into a character string
   args: buffer for answer and the length of the buffer */
{
  char *bufptr = buf;
  int len;

  *size = 0;
  
  /* insert the message-id into the buffer */
  len = strlen(vote->message_id);
  if ((*size + len + 2) > buflen) return 0; /* (size + len + space + \0) */
  sprintf(bufptr,"%s ",vote->message_id);
  *size = *size + len + 1;
  bufptr = bufptr + len + 1;  /* overwrite the \0 */

  /* insert the numbers of votes into the buffer */
  sprintf(intscratch,"%d",vote->vote_data->num_high);
  len = strlen(intscratch);
  if ((*size + len + 2) > buflen) return 0; /* (size + len + space + \0) */
  sprintf(bufptr,"%s ",intscratch);
  *size = *size + len + 1;
  bufptr = bufptr + len + 1;

  sprintf(intscratch,"%d",vote->vote_data->num_med);
  len = strlen(intscratch);
  if ((*size + len + 2) > buflen) return 0; /* (size + len + space + \0) */
  sprintf(bufptr,"%s ",intscratch);
  *size = *size + len + 1;
  bufptr = bufptr + len + 1;

  sprintf(intscratch,"%d",vote->vote_data->num_low);
  len = strlen(intscratch);
  if ((*size + len + 2) > buflen) return 0; /* (size + len + space + \0) */
  sprintf(bufptr,"%s ",intscratch);
  *size = *size + len + 1;
  bufptr = bufptr + len + 1;

  sprintf(intscratch,"%d",vote->vote_data->num_terrible);
  len = strlen(intscratch);
  if ((*size + len + 2) > buflen) return 0; /* (size + len + space + \0) */
  sprintf(bufptr,"%s ",intscratch);
  *size = *size + len + 1;
  bufptr = bufptr + len + 1;

  /* copy the ``stuff'' */
  (void) strncpy(bufptr,vote->vote_data->stuff, (buflen - *size));
  buf[(buflen - 1)] = '\0';
  len = strlen(bufptr);
  *size = *size + len;

  return 1;
}

vote_t* 
  collab_parse_vote_from_string(buf, exception)
char *buf;
int *exception;
/* take a string and parse it into a vote structure, returns NULL on failure
    *exception set to indicate extra information */
{
  vote_t *v;
  char *token;

  *exception = NORMAL;

  token = strtok(buf," \n");
  if (token == NULL) {*exception = MISSFORMED_BUFFER; return NULL;}
  if (token[0] == '.' && token[1] == '\0')
    { /* then we've got the end of data marker */
      *exception = END_OF_DATA;
      return NULL;
    }
  /* other wise create a new vote_record using the token as the message-id */
  v = collab_create_new_vote_record(token);

  token = strtok(NULL, " \n");
  if (token == NULL || !isdigit(*token)) goto failure;
  v->vote_data->num_high = atoi(token);

  token = strtok(NULL, " \n");
  if (token == NULL || !isdigit(*token)) goto failure;
  v->vote_data->num_med = atoi(token);

  token = strtok(NULL, " \n");
  if (token == NULL || !isdigit(*token)) goto failure;
  v->vote_data->num_low = atoi(token);

  token = strtok(NULL, " \n");
  if (token == NULL || !isdigit(*token)) goto failure;
  v->vote_data->num_terrible = atoi(token);

  token = strtok(NULL, "\n");
  if (token == NULL)
    v->vote_data->stuff = NULL;
  else
    { /* copy whatever stuff was left over and put it in the stuff slot */
      v->vote_data->stuff = (char *) collab_getmemory( strlen(token) + 1);
      (void) strcpy(v->vote_data->stuff,token);
    }

  return v;
  
 failure:
  free(v);
  *exception = MISSFORMED_BUFFER;
  return NULL;
}


