/* monitor.c
   generate and transmit data on the users reading patterns */

/* Copyright (c) 1993 Xerox Corporation.  All rights reserved.
 
   Use and copying of this software and preparation of derivative works
   based upon this software are permitted.  Any distribution of this
   software or derivative works must comply with all applicable United
   States export control laws.  This software is made available AS IS, and
   Xerox Corporation makes no warranty about the software, its performance
   or its conformity to any specification. */

#include <stdio.h>
#include <sys/types.h>	
#include <sys/stat.h>
#include <sys/time.h>
#include <time.h>
#include <malloc.h>
#include <string.h>
#include <vfork.h>
#include <pwd.h>
#include <errno.h>
#include "monitor.h"
#include "config.h"

/******* program global ************/
int participate_in_study = 0; /* has this user agreed to recording of session data */

/******** module globals ***********/
static group_votes *head_of_group_list = NULL;  /* global pointer to head of list */
static group_votes *cur_vote_group = NULL;      /* points to  group to be updated */
/* note, this name may clash with variables in menu.c */

/***** Internal Procs *******/
group_votes* has_vote_record_already_been_created(/* group name */);
single_vote* create_new_vote_record(/* article header */);
single_vote*  find_previous_vote_record(/* message_id */);
void send_vote_file_to_server(/* vote_file */);
void write_header_into_vote_file(/* vote_file */);
void write_all_data_into_vote_file(/* vote_file */);
void write_vote_data_into_vote_file(/* vote_file */);
void write_close_symbol_into_vote_file();
unsigned short encode_user_identity();
int add_string_to_init_file();
int are_there_any_votes();
int is_there_a_nondefault_vote();

/*========================================================================
  Handle votes
------------------------------------------------------------------------*/
void
  mon_vote_for_article(vote, message_id)
vote_type vote;
char *message_id;
{ 
  single_vote* previous_vote;
  
  if (message_id == NULL) 
    return; /* this should never happen (something wierd is up...) */
  
  if (cur_vote_group == NULL)  
    {
      printf("Error in monitor: Null group list 6- contact maltz@parc \n");
      return;
    }	
  
  if (cur_vote_group->current_vote == NULL ||
      strcmp(cur_vote_group->current_vote->message_id , message_id) != 0)
    {
      previous_vote = find_previous_vote_record(message_id);
      if (previous_vote == NULL)
	printf("Error in monitor: No article record -contact maltz@parc\n");
      else
	cur_vote_group->current_vote = previous_vote;
    }
  
  cur_vote_group->current_vote->vote = vote;
}

void
  mon_send_session_data()
/* compose a letter containing the data and mail it to
   newsfil@intrepid.xerox.com */
{
  FILE *vote_file;
  char *vote_filename = strdup("/tmp/nn-voteXXXXXX");
  int status;
  
  if (!participate_in_study) return;
  if (head_of_group_list == NULL) return; /* don't record null votes */

  vote_filename = (char *) mktemp(vote_filename);
  vote_file = fopen(vote_filename,"w");
  if (vote_file == NULL)
    {
      printf("Error in monitor: couldn't open file contact maltz@parc\n");
      return;
    }
  
  write_header_into_vote_file(vote_file);
  
  write_all_data_into_vote_file(vote_file);
  
  write_close_symbol_into_vote_file(vote_file);

  fclose(vote_file);

  send_vote_file_to_server(vote_filename);

  status = unlink(vote_filename);
  if (status != 0)
    printf("Error in monitor: couldn't delete temp file:%s",vote_filename);
}

/*========================================================================
  procedures for transporting votes
------------------------------------------------------------------------*/

void
  send_file_stream_to_shell_command(filestrm,command)
FILE *filestrm;
char *command;
/* command must be valid command that could be typed into sh(1) in the
   environment in which nn was started.  the filestrm will be sent to
   the command on stdin */
{
  FILE *new_stdin;
  long position;
  char c;
  
  new_stdin = popen(command,"w");
  if (new_stdin == NULL)
    {
      msg("Error in monitor: tapestry likeit failed - contact maltz@parc\n");
      return;
    }

  position = ftell(filestrm);	/* remember where we are in the file */
  while ((c = getc(filestrm)) != EOF)	/* copy over the file */
    putc(c,new_stdin);
  fseek(filestrm, position, 0);  /* put us back where we started in the file */
  pclose(new_stdin);
}

void 
  send_vote_file_to_server(vote_filename)
char* vote_filename;
{
  FILE *vote_file;

  vote_file = fopen(vote_filename,"r");
  if (vote_file == NULL) return;
  send_file_stream_to_shell_command(vote_file,DATA_MAILER);
  fclose(vote_file);
}
  
void
  write_header_into_vote_file(vote_file)
FILE* vote_file;
{
  time_t tloc;
  char *date;
  unsigned short user_id;

  user_id = encode_user_identity();
  tloc = time((time_t *) NULL);
  date =  ctime(&tloc);

  fprintf(vote_file,"to: newsfil@parc.xerox.com \n");
  fprintf(vote_file,"cc: maltz@parc.xerox.com \n"); 
  fprintf(vote_file,"subject: data \n");
  fprintf(vote_file,"------------ \n");
  fprintf(vote_file,"\n \n");
  fprintf(vote_file,"\n");
  
  fprintf(vote_file,"-- nn -- %u -- %s \n", user_id, date);
  free(user_id);
  free(date);
}

void 
  write_all_data_into_vote_file(vote_file)
FILE * vote_file;
{
  group_votes *group_scan;
  single_vote *vote_scan;
  int c;

  for(group_scan = head_of_group_list; group_scan != NULL; 
      group_scan = group_scan->next_group)
    {
      fprintf(vote_file,"   GROUP: %s AVAIL: %d READ: %d TIME_SCAN: %d \n",
	      group_scan->group_name, group_scan->num_articles_unread_in_group,
	      group_scan->num_articles_read, group_scan->time_spent_scanning);

      for(vote_scan = group_scan->votes ; vote_scan != NULL;
	  vote_scan = vote_scan->next_vote)
	{
	  fprintf(vote_file,"        VOTE: %d MESS-ID: %s LINES: %d TIME: %d \n",
		  vote_scan->vote,
		  vote_scan->message_id,
		  vote_scan->num_lines,
		  vote_scan->time_spent_reading);
	  free(vote_scan->message_id);
	  free(vote_scan);
	}
      
      fprintf(vote_file,"    UNREAD: ");
      for(c = 0; c < group_scan->num_articles; c++)
	{
	  fprintf(vote_file,"%d ", group_scan->length_of_articles[c]);
	}
      fprintf(vote_file,"\n");
    }
}  

void
 write_close_symbol_into_vote_file(vote_file)
FILE * vote_file;
{
  fprintf(vote_file,"\n");
}

void 
  write_vote_data_into_vote_file(vote_file)
FILE * vote_file;
{
  group_votes *group_scan;
  single_vote *vote_scan;
  int c;

  for(group_scan = head_of_group_list; group_scan != NULL; 
      group_scan = group_scan->next_group)
    {
      /* if all the votes in group_scan are default, then don't output */
      if (!is_there_a_nondefault_vote(group_scan)) continue; 

      fprintf(vote_file,"   GROUP: %s AVAIL: %d READ: %d TIME_SCAN: %d \n",
	      group_scan->group_name, group_scan->num_articles_unread_in_group,
	      group_scan->num_articles_read, group_scan->time_spent_scanning);

      for(vote_scan = group_scan->votes ; vote_scan != NULL;
	  vote_scan = vote_scan->next_vote)
	{
	  /* don't show any articles the user didn't vote for (do I really want this?)*/
	  if (vote_scan->vote == DEFAULT_VOTE) continue; 

	  fprintf(vote_file,"        VOTE: %d MESS-ID: %s LINES: %d TIME: %d \n",
		  vote_scan->vote,
		  vote_scan->message_id,
		  vote_scan->num_lines,
		  vote_scan->time_spent_reading);
	  free(vote_scan->message_id);
	  free(vote_scan);
	}

      /* is this unread data acceptable to send out?  makes it easier to figure 
	 out which arts were read at all, but I really don't think anyone would
	 take the time to sort through it, but then again, I'm naive  */
      fprintf(vote_file,"    UNREAD: ");
      for(c = 0; c < group_scan->num_articles; c++)
	{
	  fprintf(vote_file,"%d ", group_scan->length_of_articles[c]);
	}
      fprintf(vote_file,"\n");
    }
}  
	    
unsigned short
  encode_user_identity()
{
  struct passwd *pass_entry;
  unsigned short user = 0;               /* the user's identifier */
  int c;

  pass_entry = getpwuid(getuid());
  
  if (pass_entry->pw_name != NULL)
    for (c = 0; pass_entry->pw_name[c] != '\0'; c++)
      user = user * 19 + pass_entry->pw_name[c];

  if (pass_entry->pw_passwd != NULL)
    for (c = 0; pass_entry->pw_passwd[c] != '\0'; c++)
      user = user * 19 + pass_entry->pw_passwd[c];

  return user;
}
  

/*========================================================================
  Maintain the group_vote abstraction
------------------------------------------------------------------------*/
void 
  mon_setup_for_new_group(group, articles, num_articles)
group_header *group;
article_header **articles;
int num_articles;
{
  group_votes *new_group, *previous_usage;
  int cur, *art_lengths;

  previous_usage = has_vote_record_already_been_created(group->group_name);
  if (previous_usage != NULL)
    {
      cur_vote_group = previous_usage;
      return;
    }

  /* if a monitor for the group doesn't already exist, then make one */
  new_group = (group_votes *) calloc(1,sizeof(group_votes));

  /*link it up */
  new_group->next_group = head_of_group_list;
  head_of_group_list = new_group;
  cur_vote_group = new_group;

  /* init it */
  new_group->num_articles_unread_in_group = num_articles;
  new_group->group_name = strdup(group->group_name);
  new_group->num_articles_read = 0;
  new_group->time_spent_scanning = (time_t) 0;
  new_group->current_vote = NULL;
  new_group->votes = NULL;
  new_group->num_articles = 0;
  new_group->length_of_articles = NULL;

  /* record the lengths of the articles in the group */
  art_lengths =(int *) calloc(num_articles, sizeof(int));
  if (art_lengths == NULL) return;
  cur_vote_group->length_of_articles = art_lengths;
  
  for(cur = 0; cur < num_articles; cur++)
    {
      art_lengths[cur] = articles[cur]->lines;
    }
  cur_vote_group->num_articles = num_articles;
}

     
void
  mon_reading_art(ah, message_id)
article_header* ah;
char* message_id;
{
  single_vote *new_vote, *previous_vote;

  if (message_id == NULL) 
    return; /* something bad happened... (should never happen) */

  if (cur_vote_group == NULL)
    {
      printf("Error in monitor: Null group list 10- contact maltz@parc \n");
      return;
    }	

  cur_vote_group->num_articles_read++;
  /* make sure cur_vote_group->current_vote points to a proper initialized struct */
  if (cur_vote_group->current_vote == NULL ||
      strcmp(cur_vote_group->current_vote->message_id , message_id) != 0)
    {
      previous_vote = find_previous_vote_record(message_id);
      if (previous_vote == NULL)
	cur_vote_group->current_vote = create_new_vote_record(ah,message_id);
      else
	cur_vote_group->current_vote = previous_vote;
    }
}


single_vote*
  create_new_vote_record(ah, message_id)
article_header* ah;
char* message_id;
{
  /* make a new vote record and insert it into the list structure for 
     current group, it assumes cur_vote_group is set to the group_votes
     that this vote should belong to */

  single_vote *new_vote;
  
  new_vote = (single_vote *) malloc(sizeof(single_vote));

  /* link it up */
  new_vote->next_vote = cur_vote_group->votes;
  cur_vote_group->votes = new_vote;
    
  /* init it */
  new_vote->num_lines = ah->lines;
  new_vote->time_spent_reading = (time_t) 0;
  new_vote->message_id = strdup(message_id);
  new_vote->vote = (vote_type) DEFAULT_VOTE;

  return new_vote;
}

/*========================================================================
  interface to group recording reading/scanning times
------------------------------------------------------------------------*/
void
  mon_start_charging_scan_time()
{
  if (cur_vote_group == NULL)  
    {
      printf("Error in monitor: Null group list 5- contact maltz@parc \n");
      return;
    }	

  cur_vote_group->time_start_scanning = time((time_t *) NULL);
}  


void
  mon_stop_charging_scan_time()
{
  time_t now_time, elapsed_time;

  if (cur_vote_group == NULL)  
    {
      printf("Error in monitor: Null group list 6- contact maltz@parc \n");
      return;
    }	

  now_time = time((time_t *) NULL);
  elapsed_time = now_time - cur_vote_group->time_start_scanning;
  cur_vote_group->time_spent_scanning += elapsed_time;
  /* this next line is a hack to allow two consecutive stop_times
     to function properly */
  cur_vote_group->time_start_scanning = now_time;
}


void
  mon_start_charging_time_to_article()
{
  if (cur_vote_group == NULL || cur_vote_group->current_vote == NULL)
    return; /* if something is messed up, then just give up */

  cur_vote_group->current_vote->time_start_reading = time((time_t *) NULL);
}

void
  mon_stop_charging_time_to_article()
{
  time_t now_time, elapsed_time;

  if (cur_vote_group == NULL || cur_vote_group->current_vote == NULL)
    return; /* if something is messed up, then just give up */

  now_time = time((time_t *) NULL);
  elapsed_time = now_time - cur_vote_group->current_vote->time_start_reading;
  cur_vote_group->current_vote->time_spent_reading += elapsed_time;

/*  cur_vote_group->current_vote->time_start_reading = now_time; */

}

/*========================================================================
  study participation signup
------------------------------------------------------------------------*/
int 
  add_string_to_init_file(str)
char *str;
/* add the str to the beginnning of the init file;
   rtns: 0 on sucess
         -1 on failure
*/
{
  FILE *init_file_in; /* user's .nn/init file open for read */
  FILE *init_file_out; /* user's .nn/init file open for write */
  extern int errno;
  struct stat buf;
  extern char *nn_directory;
  char *path, c;

  path = mk_file_name(nn_directory,"init");
  if (stat(path,&buf) == -1)	/* see if the file exists */
    {
      if (errno != ENOENT)
	{
	  free(path);
	  return -1; 		/* can't write into init file */
	}
    }

  init_file_in = fopen(path,"r");
  unlink(path);
  init_file_out = fopen(path,"w");
  if (init_file_out == NULL)
    {
      fclose(init_file_in);
      free(path);
      return -1;
    }
  fprintf(init_file_out,"%s\n",str);	 /* actually insert the string */
  if (init_file_in != NULL)	/* if there was already an init file */
    { /* copy over the init file  */
      while ((c = getc(init_file_in)) != EOF)
	putc(c,init_file_out);
      fclose(init_file_in);
    }

  fclose(init_file_out);
  free(path);
  return 1;
}
  

void
  sign_up_a_participant()
/* set the participate in study variable, notify me of the participant */
{
  FILE *part_list; /* my list of participants file */
  char special_marker = ' ';  /* denote any problems in signing up the participant */
  unsigned short user;
  int status;

  participate_in_study = 1;	/* show this person as participating */
  
  /* add line to init file so user is a participant from now on */
  status = add_string_to_init_file("\nset participate-in-study\n");
  if (status == -1)		/* record in part_list if we can't alter the init file */
    {
      special_marker = '*';
      msg("couldn't add \"set participate-in-study\" to your ~/.nn/init file ");
    }

#ifdef NOT_DEFINED
  part_list = fopen(PARTICIPANT_FILE,"a");
  if (part_list == NULL) return;   /* we lose... */
  user = encode_user_identity();
  fprintf(part_list,"%u %c\n", user, special_marker);
  fclose(part_list);
  free(user_name);
#endif /* NOT_DEFINED */
}

/*========================================================================
  Helper routines
------------------------------------------------------------------------*/
single_vote*
  find_previous_vote_record(message_id)
char* message_id;
{   
  single_vote *scan;
  
  scan = cur_vote_group->votes;
  while (scan != NULL)
    {
      if (strcmp(scan->message_id,message_id) == 0) /* be case sensitive */
	return scan;
      scan = scan->next_vote;
    }
  return NULL;
}

group_votes*
  has_vote_record_already_been_created(name)
char* name;
{
  group_votes *scan;
  
  scan = head_of_group_list;
  while (scan != NULL)
    {
      if (strcasecmp(scan->group_name,name) == 0)
	return scan;
      scan = scan->next_group;
    }
  return NULL;
}

int 
  are_there_any_votes(grp)
group_votes *grp;
{
  while (grp != NULL)
    {  
      if (is_there_a_nondefault_vote(grp)) return 1;
      grp = grp->next_group;
    }
  return 0;
}

int
  is_there_a_nondefault_vote(grp)
group_votes *grp;
{
  single_vote *v;
  int found = 0;

  for( v = grp->votes; v != NULL ; v = v->next_vote)
    if (v->vote != DEFAULT_VOTE)
      {
	found = 1;
	break;
      }
  return found;
}

